% Copyright 2003 The University of Texas at Austin
%
% For licensing information see
%                http://www.cs.utexas.edu/users/flame/license.html 
%                                                                                 
% Programmed by: Robert van de Geijn
%                rvdg@cs.utexas.edu

function [ A_out, T_out ] = FLA_QR_UT_Accum_T_unb_var1( A, T )

  [ ATL, ATR, ...
    ABL, ABR ] = FLA_Part_2x2( A, ...
                               0, 0, 'FLA_TL' );

  [ TTL, TTR, ...
    TBL, TBR ] = FLA_Part_2x2( T, ...
                               0, 0, 'FLA_TL' );

  while ( size( ATL, 1 ) < size( A, 1 ) & ...
          size( ATL, 2 ) < size( A, 2 ) )

    [ A00,  a01,     A02,  ...
      a10t, alpha11, a12t, ...
      A20,  a21,     A22 ] = FLA_Repart_2x2_to_3x3( ATL, ATR, ...
                                                    ABL, ABR, ...
                                                    1, 1, 'FLA_BR' );

    [ T00,  t01,   T02,  ...
      t10t, tau11, t12t, ...
      T20,  t21,   T22 ] = FLA_Repart_2x2_to_3x3( TTL, TTR, ...
                                                  TBL, TBR, ...
                                                  1, 1, 'FLA_BR' );

    %------------------------------------------------------------%

    [ alpha11, ...
      a21, tau11 ] = FLA_Househ2_UT( alpha11, ...
                                     a21 );

    t01 = a10t' + A20' * a21;
    
    [ a12t, ...
      A22 ] = FLA_Apply_H_UT( tau11, a21, a12t, ...
                                          A22 );
    
    %------------------------------------------------------------%

    [ ATL, ATR, ...
      ABL, ABR ] = FLA_Cont_with_3x3_to_2x2( A00,  a01,     A02,  ...
                                             a10t, alpha11, a12t, ...
                                             A20,  a21,     A22, ...
                                             'FLA_TL' );

    [ TTL, TTR, ...
      TBL, TBR ] = FLA_Cont_with_3x3_to_2x2( T00,  t01,   T02,  ...
                                             t10t, tau11, t12t, ...
                                             T20,  t21,   T22, ...
                                             'FLA_TL' );

  end

  A_out = [ ATL, ATR
            ABL, ABR ];

  T_out = [ TTL, TTR
            TBL, TBR ];

return
